function len = faclen(nsamp, fr, mil) %{
% FACLEN - compute lengths of lag profiles
%
%  function len = faclen(nsamp, fr, mil)
%
%  Compute the lengths of the various (integer and fractional) lag profiles
%  given the number of samples (nsamp), fractionality (fr) and maximum
%  integer lag (mil).  The order of the lengths is as described in the
%  comment for ac_profmap.  I.e. for fr=4, mil=8, the result is
%  nsamp-[1 2 3 4 5 5 6 6 7 7 8 9 9 10 10 11 11 12 ... 31 31 32 33 34 35]

% 1999-08-20 TG written for making a more general ac_profmap()

% Since the desired vector is increasing in steps of either 0 or 1, we
% will construct this vector using cumsum of a vector containing the
% increments between successive elements.

len = nsamp - ....			% as sketched above
    cumsum( [ ...
	ones(1,fr-1), ...		% begin with fr-1 elements used once
	kron(ones(1,mil-1), [ ...	% then mil-1 times the pattern
	     1 ...			% of a number used just once
	     kron(ones(1,fr-1), ...	% then fr-1 numbers  
		 [1 0]) ...		% used twice
	]) ...				% end of (mil-1) pattern
	ones(1,fr)]);			% then fr elements used once
%}faclen

