function h = pp_axis(Axnumber, Action)

%PP_AXIS Delete/Create various pp-generic figure axis.
%
%   PP_AXIS(axnumber, Action)
%
%   7-Dec-1999 Jm

%{--------------------------------------------------------------

    global pp_USERHDR pp_USERFIG1 pp_USERFIG2 pp_USERFIG3

    if nargin > 2
        error('Wrong number of input arguments');
    end        

    %-------------------------------------------------------------------------
    % Position of the various axis within the figure window, normalized units.
    %-------------------------------------------------------------------------

    header      = [0.20 0.77 0.75 0.14];     % AX 0: header

    single      = [0.10 0.10 0.85 0.64];     % AX 1: axis for profiles

    panel_major = [0.10 0.10 0.64 0.64];     % AX 2: axis for primary plots in ACF/SP
    panel_minor = [0.80 0.10 0.15 0.64];     %       axis for secondary plot in ACF/SP    

    panel_left  = [0.15 0.10 0.35 0.64];     % AX 3: left panel in twopanel plots
    panel_right = [0.60 0.10 0.35 0.64];     %        right panel in twopanel plots
    
    panel_left  = [0.25 0.10 0.30 0.64];     % AX 3: left panel in twopanel plots
    panel_right = [0.65 0.10 0.30 0.64];     %        right panel in twopanel plots

 
    if nargin == 0
        %-------------------------------------------------
        % Return current axis handle(s)
        %-------------------------------------------------

        h = [];
        if ~isempty(pp_USERHDR),  h = [h pp_USERHDR];  end;
        if ~isempty(pp_USERFIG1), h = [h pp_USERFIG1]; end;
        if ~isempty(pp_USERFIG2), h = [h pp_USERFIG2]; end;
        if ~isempty(pp_USERFIG3), h = [h pp_USERFIG3]; end;
 
    elseif nargin == 1
        %-------------------------------------------------
        % Return axis handle(s)
        %-------------------------------------------------

        if Axnumber == 0
            h = pp_USERHDR;
        elseif Axnumber == 1
            h = pp_USERFIG1;
        elseif Axnumber == 2
            h = pp_USERFIG2;
        elseif Axnumber == 3
            h = pp_USERFIG3;
        else
            error('Illegal input parameter');
        end            

    elseif strcmp(Action,'init') | strcmp(Action,'create')
        %-------------------------------------------------
        % Create
        %-------------------------------------------------
    
        if Axnumber == 0
            % Create header axis

            if isempty(pp_USERHDR)
                pp_USERHDR = axes('pos',header);
                set(pp_USERHDR,'xtick',[]);
                set(pp_USERHDR,'ytick',[]);
                set(pp_USERHDR,'box','on');
                set(pp_USERHDR,'color',[1 1 1]); 
            end
            h = pp_USERHDR;

        elseif Axnumber == 1
            % Create profile axis

            delete_axis([2 3]);

            if isempty(pp_USERFIG1)
                pp_USERFIG1 = axes('pos',single);
                set(pp_USERFIG1,'box','on');
            end
            h = pp_USERFIG1;

        elseif Axnumber == 2
            % Create ACF/SP major and major axis

            delete_axis([1 3]);

            if isempty(pp_USERFIG2)
                pp_USERFIG2(1) = axes('pos',panel_major);
                set(pp_USERFIG2(1),'box','on');

                pp_USERFIG2(2) = axes('pos',panel_minor);
                set(pp_USERFIG2(2),'box','on','ytick',[]);
            end
            h = pp_USERFIG2;

        elseif Axnumber == 3   
            % Create two equal size axis

            delete_axis([1 2]);
            
            if isempty(pp_USERFIG3)
                pp_USERFIG3(1) = axes('pos',panel_left);
                set(pp_USERFIG3(1),'box','on');

                pp_USERFIG3(2) = axes('pos',panel_right);
                set(pp_USERFIG3(2),'box','on','ytick',[]);
            end
            h = pp_USERFIG3;
        else
            error('Illegal input parameter');
        end

    elseif strcmp(Action,'delete')  | strcmp(Action,'done')
        %-------------------------------------------------
        % Delete
        %-------------------------------------------------
  
        if Axnumber == 0
            % Delete header axis
            if ~isempty(pp_USERHDR)
                delete(pp_USERHDR);
                pp_USERHDR = [];
            end                

        elseif Axnumber == 1
            % Delete profile axis
            if ~isempty(pp_USERFIG1)
                delete(pp_USERFIG1);
                pp_USERFIG1 = [];
            end                

        elseif Axnumber == 2
            % Delete major & minor axis
            if ~isempty(pp_USERFIG2)
                delete(pp_USERFIG2(1));
                delete(pp_USERFIG2(2));
                pp_USERFIG2 = [];
            end                

        elseif Axnumber == 3   
            % Delete left and right axis
            if ~isempty(pp_USERFIG3)
                delete(pp_USERFIG3(1));
                delete(pp_USERFIG3(2));
                pp_USERFIG3 = [];
            end

        elseif Axnumber == Inf
            % Delete all axis
            delete_axis([0 1 2 3]);
        else
            error('Illegal input parameter');
        end
        h = [];

    else
        error('Illegal input argument')        
    end

%}pp_axis    


function delete_axis(axlist)
%---------------------------
%{
    for ax = axlist
        pp_axis(ax,'delete');
    end
%}delete_axis            
