function pp_call(Action,varargin)

% PP_CALL(ACTION,VARARGIN)
%
% pp_generic GUI callbacks or functions called from the callbacks.
% For the actual plotting functions, see pp_generic.m
%
% See also pp_gui.m, where the GUI is defined.
%
% 26-Oct-1999 Jm

%    Action 
%----------------------------------------------------------------------
%<  measure  >
%<  options  >


%{
    % These globals are (also) used by rtg_main. All other rtg_XXX globals
    % should be renamed to pp_XXX, or something:

    if nargin < 1
        error('Wrong number of input arguments')
    end        

    global ...
        USERPLOT_FUNC ...
        d_data ...
        rtg_ERRMSG ...
        rtg_FIGhandle ...
        rtg_USERERR

    stdout = 1;

    if 0
        ;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%<  measure  >%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %% Compute stddev of currently selected line(s)
    %%  
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    elseif strcmp('std_line',Action) ...
           | strcmp('std2_line',Action) ...
           | strcmp('minmax_line',Action) ...
           | strcmp('capture_line',Action) ...
           | strcmp('mean_line',Action) ...
           | strcmp('select_line',Action) ...
           | strcmp('get_xy',Action) ...
           | strcmp('fit',Action) 
    %{
        rtg_measure(Action,varargin{:})
    %}measure


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%<  options  >%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %% Set (and view) options of user plots
    %% 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    elseif strcmp('options',Action) %{
        global pp_OPT GATE
        
        if isempty(pp_OPT), return, end 
        fig = rtg_FIGhandle(2);
%         ppwarning(fig, 'GUI not available -- use command ''ppopt''')
%         return;
 
        opt  = pp_OPT;
        pp_gate = GATE; 
        fprintf(stdout, '<options>...');
        pp_setopt_gui(varargin{1}); % display pp_options GUI

        if (~isequal(pp_OPT,opt)|~isequal(GATE,pp_gate)) & ~isempty(d_data)
            pp_call('userplot');
        end
        fprintf(stdout,'\n>> ');
        
%         opt = rtg_OPT;
%         fprintf(stdout, '<options>...');
%         msg = setopt2;
%         if ~isempty(msg)
%             fprintf(stdout,'\b\b\b %s ', msg);
%         else
%             fprintf(stdout,'\b\b\b     ');
%         end
%         if ~isequal(rtg_OPT,opt) & ~isempty(d_data)
%             pp_call('userplot');
%         end
%         fprintf(stdout,'\n>> ');    
    %}options    


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%<  resize  >%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %% Change window size
    %%
    %% varargin{1}: resize mode 'R'= reset, 'L'=Large,  etc
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    elseif strcmp('resize',Action) %{
        global pp_FIGpos rtg_channelMU rtg_channelMU2 rtg_modeMU
    
        if nargin ~= 2
            error('Wrong number of input parameters');
        end            
        fig = gcf;
        orig_units = get(fig,'Units'); 
        
        % Resize the window proper
        
        rtg_resizewin(fig, pp_FIGpos, varargin{1});
        
        % Make sure that the channel and mode menus stay on correct place
        
        uitotop(fig, rtg_channelMU, 22, 0.00, 0.25)
        if ~isempty(rtg_channelMU2)
            uitotop(fig, rtg_channelMU2, 22, 0.25, 0.25)
            uitotop(fig, rtg_modeMU, 22, 0.75, 0.25)
        end    
        uitotop(fig, rtg_modeMU, 22, 0.50, 0.25)
        set(fig,'Units',orig_units); 
        
        % Enforce an explic draw, it appears that a mere "drawnow" leaves
        % header text to wrong size.
        
        if ~isempty(d_data)
            drawnow;
            pp_call('userplot');                
        end
    %}resize


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%<  setchannel  >%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %% Set the primary user plot data channel and update the plot.
    %%
    %% varargin1: current index to the channel mu
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    elseif strcmp('setchannel',Action) %{
        global rtg_viewMU2_fz rtg_channelMU rtg_CHANNELS
        global FREQCH
        
        oldchannel = FREQCH;
        FREQCH = varargin{1};
        ch_name = strip(rtg_CHANNELS(FREQCH).name);
        fprintf(stdout,'<ch> %s ...',ch_name);
        rtg_USERERR = 0;
        pp_call('userplot');
        if rtg_USERERR
            set(rtg_channelMU,'value',oldchannel);
            FREQCH = oldchannel;
            fprintf('\b\b\billegal (now)\n>> ');
        else    
            fprintf('\b\b\b      \n>> ');
        end    
    %}setchannel


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%<  setchannel2  >%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %% Set the secondary user plot data channel. Force axis to autoscale.
    %% varargin{1}: channelno
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    elseif strcmp('setchannel2',Action) %{
        global rtg_viewMU2_fz rtg_CHANNELS2 FREQCH2

        rtg_USERERR = 0;
        orig_ch2 = FREQCH2;
        FREQCH2 = varargin{1};
        rtg_USERAX= [];
        set(rtg_viewMU2_fz,'check','off');
        fprintf(stdout, '<secondary ch> %s ...', ...
                         rtg_CHANNELS2(FREQCH2).name );
        pp_call('userplot');
        if rtg_USERERR
            FREQCH2 = orig_ch2;
            pp_call('userplot');
        else            
            fprintf('\b\b\b    \n>> ')
        end            
    %}setchannel2


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%<  setmode  >%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %% Set user plot mode. This also forces axis to autoscale.
    %%
    %% varargin{1}: Current index into the Mode menu
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    elseif strcmp('setmode',Action) %{
        global ...
            rtg_viewMU2_fz rtg_MODES rtg_modeMU rtg_USERAX ...
            MODE rtg_USERFIG rtg_CLIM

        oldmode = MODE;
        MODE = varargin{1};
        rtg_USERAX = [];
        set(rtg_USERFIG,'CLimMode','auto'); 
        rtg_CLIM = [];
        set(rtg_viewMU2_fz,'check','off');
        modename = strip(rtg_MODES(varargin{1}).name);
        fprintf(stdout,'<mode> %s ...',modename);
        rtg_USERERR = 0;
        pp_call('userplot');
        if rtg_USERERR
            set(rtg_modeMU,'value',oldmode);
            MODE = oldmode;
            fprintf('\b\b\billegal (now)\n>> ');
        else    
            fprintf(stdout,'\b\b\b     \n>> ')
        end
    %}setmode


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%<  userax_freeze  >%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %% Read current user figure azis to the global rtg_USERAX. 
    %% Update menu check mark.
    %%  
    %% varargin{1}: freezeMU handle
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    elseif strcmp('userax_freeze',Action) %{
        global rtg_USERAX rtg_CLIM rtg_USERFIG

        check = get(varargin{1},'check');
        set(0,'CurrentFigure',rtg_FIGhandle(2))
        if strcmp(check,'off') % change to fixed scale
            rtg_USERAX = axis;
            rtg_CLIM = get(rtg_USERFIG, 'CLim');
            set(rtg_USERFIG, 'CLim', rtg_CLIM);
            get(rtg_USERFIG, 'CLimMode');
            set(varargin{1},'check','on');
            fprintf('<freeze> axis scale fixed\n>> ')';
        else % change to autoscale
            rtg_USERAX = [];
            set(rtg_USERFIG, 'CLimMode', 'auto');
            rtg_CLIM = [];
            set(varargin{1},'check','off');
            if ~isempty(findobj(rtg_USERFIG,'Type','image'))
                axis tight;
            elseif ~isempty(findobj(rtg_USERFIG,'Type','surface'))
                axis tight;
            else        
                xax;
                axis auto;
                drawnow;
            end                
            fprintf('<freeze> axis auto-scaled\n>> ')';
        end    
    %}userax_freeze


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%<  userplot  >%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    elseif strcmp('userplot',Action) %{
        global  rtg_h rtg_USERHDR rtg_USERFIG

        pperr('');
        feval(USERPLOT_FUNC,'go');
        if pperr
        
            % Clear user plot window, to force the user to try something
            % that will NOT give an error.
           
            ax = pp_axis;
            for h = ax 
                axes(h);
                cla;
                set(h,'xtick',[],'ytick',[]); 
            end

            if pperr
                pp_warning(rtg_FIGhandle(2),pperr);
            else
                pp_warning(rtg_FIGhandle(2),'????');
            end
            pperr('');                                
        end            
        rtg_h = [];
    %}userplot


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%<  zoomx  >%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %% Get and set x-axis area on user plots
    %%  
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    elseif strcmp('zoomx',Action) %{
        global rtg_USERAX rtg_viewMU2_fz

        if nargin == 1
            figure(rtg_FIGhandle(2));
            fprintf('\r<select x> {drag x area} ...');
            set(gcf, 'interrupt', 'on',...
                     'windowbuttondown', 'pp_call(''zoomx'',1)',...
                     'windowbuttonup', '1;',...
                     'pointer', 'cross' );
        elseif varargin{1} == 1    
            figure(rtg_FIGhandle(2));
            set(gcf,'pointer','arrow');
            ax = axis;
            p1 = get(gca,'currentpoint'); p1= p1(1,1:2);
            xy = get(gcf,'currentpoint');
            rbbox([xy 0 0],xy)
            p2 = get(gca,'currentpoint'); p2= p2(1,1:2);
            xy2 = get(gcf,'currentpoint');
            set(gcf,'windowbuttondown','',...
                    'windowbuttonup','',...
                    'interrupt','off',...
                    'pointer','arrow');
            if ( abs(xy(1)-xy2(1)) + abs(xy(2)-xy2(2)) ) < 11
                fprintf('\r<select> Cancelled               \n>> '); 
            else   
                x1 = min([p1(1),p2(1)]);
                x2 = max([p1(1),p2(1)]);
                y1 = min([p1(2),p2(2)]);
                y2 = max([p1(2),p2(2)]);
                if y1*y2 > 0
                    h= findobj(gca,'Tag','xax');
                    if ~isempty(h)
                        delete(h)
                    end
                end        
                set(gca,'xlim',[x1,x2],'xlimmode','manual');
                set(gca,'ylimmode','auto');
                drawnow;
                rtg_USERAX = axis;
                set(rtg_viewMU2_fz,'check','on');
                fprintf('\r<select> axis freezed           \n>> ');
            end     
        end
    %}zoomx


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%<  zoomxy  >%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %% Get, set and update x-axis and y-axis area on user plot
    %%  
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    elseif strcmp('zoomxy',Action) %{
        global rtg_USERAX rtg_viewMU2_fz

        if nargin == 1
            figure(rtg_FIGhandle(2));
            fprintf(stdout,'\r<select xy> {drag xy box} ...');
            set(gcf,'interrupt','on',...
                    'windowbuttondown','pp_call(''zoomxy'',1)',...
                    'windowbuttonup','1;',...
                    'pointer','cross' );
        elseif varargin{1} == 1    
            figure(rtg_FIGhandle(2));
            ax = axis;
            p1 = get(gca,'currentpoint'); p1= p1(1,1:2);
            xy = get(gcf,'currentpoint');
            rbbox([xy 0 0],xy)
            p2 = get(gca,'currentpoint'); p2= p2(1,1:2);
            xy2 = get(gcf,'currentpoint');
            set(gcf,'windowbuttondown', '',...
                    'windowbuttonup', '',...
                    'interrupt', 'off',...
                    'pointer', 'arrow' );
            if ( abs(xy(1)-xy2(1)) + abs(xy(2)-xy2(2)) ) < 11
                fprintf(stdout,'\r<select> Cancelled               \n>> '); 
            else
                x1 = min([p1(1),p2(1)]);
                x2 = max([p1(1),p2(1)]);
                y1 = min([p1(2),p2(2)]);
                y2 = max([p1(2),p2(2)]);
                set(gca,'xlim',[x1,x2],'xlimmode','manual');
                set(gca,'ylim',[y1,y2],'ylimmode','manual');
                drawnow;
                rtg_USERAX = axis;
                global rtg_viewMU2_fz
                set(rtg_viewMU2_fz,'check','on');
                fprintf(stdout,'\r<select xy> axis freezed       \n>>');
            end    
        end
    %}zoomxy


    end

%}pp_call

function pp_warning(fig,msg) %{
%---------------------------
    global USERPLOT_FUNC
    warningdlg(msg,' Warning',fig);
%}rtgwarning    
