function result = pp_generic(Action,Param) %{

%PP_GENERIC Plot power profiles (etc) of gup0,gup3 and tau0 MAT dumps.
% based on pp_gup3.m  
%   Input is via global variables.
% version 1.0 of the "Gup 3 Software"
% 08 April 1999: Arnaud Masson
% 07 July 1999 Jm:  
%       Version recognization added. 
%       Some individual global variables moved to the GUP variable.
%       Range calculations changed.
%       Alternating code spectrum calculation changed.
% 12-Oct-99 Am: lot of modifications to turn it to generic
% 26-Oct-99 Jm: More modifications for rtg 2.0 compability: rtg_OPT 
%               replaced by pp_OPT, seterr replaced by pperr, references
%               to rtg_CONST replaced by function rtgconst().
% 5-Nov-1999 Jm: Some internal functions moved to stand_alone functions.
%----------------------------------------------------------------------------

rtg_debug(2,sprintf('%s(%s)',mfilename,Action));

global d_data		 % load with MAT raw data file

global AC_d_data	 % New Parameter (version 1.0)
global d_ExpInfo	 % load with MAT raw data file

global CALTEMP      	 % caltemp=95  cf RTG.main
global MODE		 % defined in gdef
global FREQCH	   	 % defined in gdef
global GATE      	 % gate number for ACF and SPEC
			 % defined in gdef and in rtg_main.m
			 % , see also pp_init at EOF
global DMPT		 % defined in rtg_main or dirm
global rtg_FIGhandle 	 % handles of rtg figure findows, dirm
global rtg_USERFIG   	 % handle of plot axis on user plots,dirm
global rtg_USERAX    	 % scaling for the axis. use auto if = []., dirm
global rtg_MODES    	 % dirm, see also pp_init at EOF 
global rtg_CHANNELS  	 % dirm, see also pp_init at EOF
global GUP	         % defined in this file
global GAIN         	 % defined in this file
global pp_OPT	     	 % dirm, see also pp_init at the EOF
global ELEV		 % dirm
global GDEFNAME	
global old_ExpInfo

pperr('');

if strcmp(Action,'go')
    ;
elseif strcmp(Action,'init')
    pp_generic_init(Param);
    result = pperr;
    rtg_debug(2,sprintf('%s done',mfilename));
    return;
elseif strcmp(Action,'config')
    result = 'pp_config';
    return;
else
    pperr(['Illegal pp_generic command (' Action ')']);
    return
end    
  
% Find the version of the experiment.
% If the version has changed, reinitialize.

[isnew, expid, version, old_ExpInfo, msg] = check_version(old_ExpInfo);
if isempty(isnew)
    pperr(msg);
    return
elseif isnew
    fprintf(1,'\n\nInitialising for %s ...\n', old_ExpInfo);
    fprintf(1,'gdef: %s\n', GDEFNAME);
    msg = feval(GDEFNAME,'reinit',version);
    if ~isempty(msg)
        pperr(msg);
        return 
    end 
    if exist('pp_config') == 2
        fprintf(1,'pp_config: %s\n', which('pp_config'));
        pp_config('reinit',expid,version);
    else
        fprintf(1,'pp_config: -\n');
    end        
end    

% Get the interactively defined RMB number and plot mode's ID

[RMB,ID] = pp_xxx_checks; 
if isempty(RMB)
    rtg_debug(2,sprintf('  %s done(error)',mfilename));
    return 
end

% If userfig 2 is in use, delete it from those plots that do not need it :

if ID < 10,  
    rtg_userfig2('delete'); 
end

% Ensure that we are going to draw to the right place

set(rtg_FIGhandle(2),'CurrentAxes',rtg_USERFIG);
set(0,'currentfigure',rtg_FIGhandle(2));


if ID == 1 %{%%%%%%%%%%%%%%%%[ s,b ] %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if ismulti(RMB)
        pperr('Multiple channels not allowed (now)');
        return
    end
    
    if RMB==[4 8]
        pperr('Channels 4 and 8 don''t have the same duration'); 
        return
    end

    if isLP(RMB)
        sig  = spp(RMB);
    else
	sig  = spp(RMB);    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            % [w,tail] = wpp(RMB);  %NEED TO BE IMPLEMENTED !!!!!!! 
                            % sig  = sig./w;        %FOR BOTH tau0 AND gup3,16/10/99
    end                     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    bgr1 = bpp(1,RMB);

    if exist('GUP.b2sa') 
        bgr2 = bpp(2,RMB);
    end
    r = ppr(RMB);
 
    xlbl = 'Range';
    if strcmp(ppopt('altid'),'yes')  
        r = range2alt(r,ELEV);
        xlbl = 'Altitude [km]';
    end
    
    if isLP(RMB)
    	if exist('GUP.b2sa')
    	    plot(r,bgr1,'y',r,bgr2,'c',r,sig,'m');
        else
            plot(r,bgr1,'y',r,sig,'m');
        end
    else						 %%%%%%%%%%%%%%%%%%
%	plot(r,bgr1,'y',r,sig,'m',r(tail),sig(tail),'r');% TO BE KEPT
	plot(r,bgr1,'y',r,sig,'m'); 			 % TO BE DESTROYED
    end    	 	 				 %%%%%%%%%%%%%%%%%%

    annot(xlbl,'Power','',rtgconst('chmean'));


elseif ID == 2 	%}{----------------[ s,<b> ]-------------------------------
   		%  only used by gup0 experiment, 14/10/99, Am

    if RMB==[4 8] 
        pperr('Channels 4 and 8 don''t have the same duration'), return
    end

   
    if ismulti(RMB) 
        pperr('multiple channels not allowed (now)'), return
    end


    sig = spp(RMB);
    bg  = 0.5*( bpp(1,RMB) + bpp(2,RMB) );
    r   = ppr(RMB);
    xlbl = 'Range';
    if strcmp(ppopt('altid'),'yes')
        r = range2alt(r,ELEV);
        xlbl = 'Altitude [km]';
    end  
    plot (  ...
          r, bg, 'c',...
          r,sig, 'm'...
       )
    annot(xlbl,'power','',rtgconst('chmean'));



elseif ID == 3 %}{-------------[ s - b ] or---[ s - <b> ]-----------------
   
    test=rtg_CHANNELS(RMB(1)); % cf ID==1   
    
    if RMB==[4 8] 
        pperr('Channels 4 and 8 don''t have the same duration') 
        return
    end

    if ismulti(RMB)
        oldhold= ishold;

        for rmb = RMB;

% sig
%----
	    if test.isLP
    			  sig  = spp(rmb);
            else
			  sig  = spp(rmb);  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%		     [w,tail] = wpp(rmb);   %NEED TO BE IMPLEMENTED !!!!!!! 
%		      	  sig  = sig./w;    %FOR BOTH tau0 AND gup3,16/10/99
            end				    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% bgr
%----           
            if ~exist('GUP.b2sa'),bgr = bpp(1,rmb);
            else
               			  bgr = 0.5*(bpp(1,rmb)+bpp(2,rmb));
            end
% r
%----
            r = ppr(rmb);
% pow
%----
            if test.isLP
                pow = ppscaler(rmb, r) .* (sig-bgr);
            else
                % no background subtraction in AC data
                pow = ppscaler(rmb, r) .* sig;
            end
                
            if strcmp(ppopt('altid'),'yes')
                r = range2alt(r,ELEV);
                xlbl = 'Altitude [km]';
            else        
                xlbl = 'Range [km]';
            end

            plot( r, pow, GUP.ltype{rmb})
            hold on
        end    
        if ~oldhold, hold off, end

    else

% sig
%----
	    if test.isLP
    			  sig  = spp(RMB);
            else
			  sig  = spp(RMB);  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%		     [w,tail] = wpp(RMB);   %NEED TO BE IMPLEMENTED !!!!!!! 
%		      	  sig  = sig./w;    %FOR BOTH tau0 AND gup3,16/10/99
            end				    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% bgr
%----           
            if ~exist('GUP.b2sa'),bgr = bpp(1,RMB);
            else
               			  bgr = 0.5*(bpp(1,RMB)+bpp(2,RMB));
            end
% r
%----
            r = ppr(RMB);
% pow
%----            
            if test.isLP
            	pow = ppscaler(RMB(1),r) .* (sig-bgr);
            else
            	pow = ppscaler(RMB(1),r) .* sig;
            end
            
            if strcmp(ppopt('altid'),'yes')
            	r   = range2alt(r,ELEV);
               xlbl = 'Altitude [km]';
            else        
               xlbl = 'Range [km]';
            end

           plot( r, pow, 'm')
    end

    if strcmp(ppopt('range_scaling'),'yes')
       
       if test.isLP % cf remark at the beginning of ID==1
 
            annot( xlbl, 'Power \cdot r ^2', '', rtgconst('chmean'))        
        else
            lag = round(GUP.Tau*1e6);	% used to be called GUP.Tau   (tau0)
            annot(  xlbl, ...			%  and      GUP.Tau (gup3)
                    sprintf('Lag %d\\mus  \\cdot r ^2', lag), ...
                    '', ...
                    rtgconst('chmean'))        
        end
            
    else
        annot(xlbl, 'Power','',rtgconst('chmean'))        
    end        


elseif ID == 4 %}{---------------[ s ]------------------------------------
   
test=rtg_CHANNELS(RMB(1));   % 12-Oct-1999 Am: added to suppress function isLP
			     % in order to provide a pp_generic
   if RMB==[4 8] 
        pperr('Channels 4 and 8 don''t have the same duration'), return
    end


    if ismulti(RMB)
        oldhold= ishold;
        for rmb = RMB;
% r
%----
            r = ppr(rmb);
            if strcmp(ppopt('altid'),'yes')
                r = range2alt(r,ELEV);
                xlbl = 'Altitude [km]';
            else        
                xlbl = 'Range [km]';
            end
% sig
%----
	    if test.isLP
    			  sig  = spp(rmb);
            else
			  sig  = spp(rmb);  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%		     [w,tail] = wpp(rmb);   %NEED TO BE IMPLEMENTED !!!!!!! 
%		      	  sig  = sig./w;    %FOR BOTH tau0 AND gup3,16/10/99
            end				    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


            plot( r, sig, GUP.ltype{rmb})
            hold on
        end    
        if ~oldhold, hold off, end

    else
            
% r
%----
        r = ppr(RMB);
        if strcmp(ppopt('altid'),'yes')
            r = range2alt(r,ELEV);
            xlbl = 'Altitude [km]';
        else        
            xlbl = 'Range [km]';
        end
% sig
%----
	    if test.isLP
    			  sig  = spp(RMB);
            else
			  sig  = spp(RMB);  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%		     [w,tail] = wpp(RMB);   %NEED TO BE IMPLEMENTED !!!!!!! 
%		      	  sig  = sig./w;    %FOR BOTH tau0 AND gup3,16/10/99
            end				    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


        plot( r, sig, 'm')

    end
    if test.isLP
       annot(xlbl,'Power','',rtgconst('chmean'))        
    else
        lag = round(GUP.Tau*1e6);	% used to be called GUP.Tau (tau0)
        annot( xlbl, ...				% GUP.tauLP (gup3)
                sprintf('Lag %d\\mus', lag), ...
                '',rtgconst('chmean'))        
    end        

elseif ID == 4.5 %}{------------------[x]--------------------------------
		 % only used by tau0 experiment, 14/10/99, Am

    if ismulti(RMB) 
        pperr('multiple channels not allowed (now)'), return
    end

    b1 = xpp(RMB);
    b = 0:length(b1)-1;
    plot( b,b1,'y');
    annot('Address','0-lag','',rtgconst('chmean'))

elseif ID == 5 %}{--------------------[b,c]-or-[b1,c1,b2,c2]-----------------

    if ismulti(RMB) 
        pperr('multiple channels not allowed (now)'), return
     end
     
test=rtg_CHANNELS(RMB(1));
    if test.isLP
        b = GUP.pp-1;
    else
        b = GUP.pp(1:end-1) - 1;    
    end        
    c = b(end) + GUP.cpp;

    b1 = bpp(1,RMB);
    c1 = calpp(1,RMB);
    if exist('GUP.b2sa')
    b2=bpp(2,RMB);
    c2=cpp(2,RMB);
    end
 
    if ~exist('GUP.b2sa')
    plot(...
         b,b1,        'y', ...
         c,c1,        'ro' ...
         )
    else
      plot(...
         b,b1,        'y', ...
         c,c1,        'yo', ...
         b,b2,        'c', ...
         c,c2,        'co'  ...
         )
    end
    annot('Address','Power','',rtgconst('chmean'))

elseif ID == 5.5 %}{--------------------[b]--------------------------------
   		 % only used by tau0 experiment, 14/10/99, Am
   		 % previously called ID=5 in pp_tau0

    if ismulti(RMB) 
        pperr('multiple channels not allowed (now)'), return
    end

    b1 = bpp(1,RMB);
    b = GUP.pp(1:end-1) - 1;    
    plot( b,b1,'y');
    annot('Addres','Power','',rtgconst('chmean'))


elseif ID == 6 %}{-----------------[ b,c,T_sys ]----------------------------
   
test=rtg_CHANNELS(RMB(1));   % 12-Oct-1999 Am: added to suppress function isLP
			     % in order to provide a pp_generic

    if test.isLP
                              
       if GUP.sbweight==4,
          pp = GUP.pp(3:end);   % only valid for gup0, 14/10/99, Am
          cpp= GUP.cpp(3:end-1);% only valid for gup0, 14/10/99, Am
       else
          pp = GUP.pp(1:end);
          cpp= GUP.cpp(1:end);
       end
       
    else
       pp = GUP.pp(1:end-1);
       cpp= GUP.cpp(1:end);
    end        
    
    if ismulti(RMB)
        for rmb = RMB	
           if exist('GUP.b2sa')  
            	bgr = 0.5 *( bpp(1,rmb) + bpp(2,rmb));
            	cal = 0.5 *( calpp(1,rmb) + calpp(2,rmb));
           else       
        	  	bgr = bpp(1,rmb);
            	cal = calpp(1,rmb);
            end
            
            if GUP.sbweight==4
             ave_b = mean(bgr(pp));
             ave_c = mean(cal(cpp)) - ave_b;  
            else
             ave_b = mean(bgr(end-100:end));
             ave_c = mean(cal) - ave_b;
            end
            systemp(rmb)= round(( ave_b / ave_c ) * CALTEMP);
        end
        h = stem(RMB,systemp,'r:','fill');
        set(h(1),'MarkerFacecolor','y','Markersize',10);        
        annot( 'channel', 'systemp [ K ]', ...
            sprintf('T[%dK-%dK]',min(systemp),max(systemp)),rtgconst('chmean') )
	 
    else
       if exist('GUP.b2sa')  
        bgr = 0.5 *( bpp(1,RMB) + bpp(2,RMB));
        cal = 0.5 *( calpp(1,RMB) + calpp(2,RMB));
       else       
        bgr = bpp(1,RMB);
        cal = calpp(1,RMB);
       end
     
        b   = pp-1;
        c   = pp(end) + cpp;
        plot (...
	        b, bgr(pp), 'g-', ...
	        c, cal(cpp), 'ro' ...
           )
        if GUP.sbweight==4,
           ave_b= mean(bgr(pp));
           ave_c = mean(cal(cpp)) - ave_b;
        else
           ave_b = mean(bgr(end-100:end));
           ave_c = mean(cal) - ave_b;
        end
                systemp = round(( ave_b / ave_c ) * CALTEMP);
        annot('Address', 'Power', sprintf('T=%dK',systemp),rtgconst('chmean'))
	 
    end


elseif ID == 7 %}{---------------------[ tx ]----------------------------

    oldhold = ishold;
    for rmb = RMB;
        [z,t] = tx(rmb);
        x = real(z);
        y = imag(z);
        N=GUP.txsamples; % previously N= GUP.Nfract*GUP.Nbits, 14/10/99, Am
        a=1:N;
	plot (...
	     t(a),x(a),   'y-o', ...
	     t(a),y(a),   'm-o' ...
	     )
        hold on
    end    
    if ~oldhold, hold off, end
    annot('time [ \mus ]','Real & Imag','',rtgconst('chmean'))        


elseif ID == 7.1 %}{---------------------[ re tx ]----------------------------
		 % only used by tau0 experiment, 14/10/99, Am

    oldhold = ishold; 
    color = {'c','y'};
    for rmb = RMB;
        [z,t] = tx(rmb);
        x = real(z);
        pow = mean(abs(z) .^2 )/1e6;
        n1 = 1;
        N = GUP.Nfract*GUP.ACnbits;
        n2 = N;
        for cycle = 1:GUP.Ncycles
            a = n1:n2;
            plot(a-1,x(a),color{mod(cycle,2)+1});
            hold on;
            n1 = n2+1;
            n2 = n1 + N - 1;
        end            
    end    
    if ~oldhold, hold off, end
    annot('Addr','Real part',...
        sprintf(' Pow=%1.2f',pow),rtgconst('chmean'))        


elseif ID == 7.2 %}{---------------------[ im tx ]----------------------------
		 % only used by tau0 experiment, 14/10/99, Am

    oldhold = ishold; 
    color = {'g','w'};
    for rmb = RMB;
        [z,t] = tx(rmb);
        x = imag(z);
        n1 = 1;
        N = GUP.Nfract*GUP.ACnbits;
        n2 = N;
        for cycle = 1:GUP.Ncycles
            a = n1:n2;
            plot(a-1,x(a),color{mod(cycle,2)+1});
            hold on;
            n1 = n2+1;
            n2 = n1 + N - 1;
        end            
    end    
    if ~oldhold, hold off, end
    annot('Addr','Imag part','',rtgconst('chmean'))        




elseif ID == 8 %}{---------------------[ abs tx ]------------------------
	       % only used by gup0 and gup3 experiment, 14/10/99, Am

  oldhold= ishold;
     for rmb= RMB;
        [z,t]=tx(rmb);
	     plot (...
	     t, abs(z), GUP.ltype{rmb} ...
	     )
        hold on
    end    
    if ~oldhold, hold off, end
    annot('time [ \mus ]', 'magnitude','',rtgconst('chmean'))        

elseif ID == 8.5 %}{---------------------[ abs tx ]------------------------
		 % only used by tau0 experiment, 14/10/99, Am

    oldhold= ishold;
    linetype = {'y-o','c-o'};
    for rmb= RMB;
        [z,t]=tx(rmb);
        N = GUP.Nfract*GUP.ACnbits;
        a = 1:N;
        for k = 1:2
	    plot (...
	         a-1, abs(z(a)), linetype{k} ...
	         )
            hold on
            a = a + N;
                        
        end
    end    
    if ~oldhold, hold off, end
    grid
    annot('Addr', 'Magnitude','',rtgconst('chmean'))        




elseif ID == 9 %}{-------------------[ phase tx ]------------------------
	       % only used by gup0 and gup3 experiment, 14/10/99, Am

    oldhold= ishold;
    for rmb= RMB;
        [z,t]=tx(rmb);
        z= angle(z) * 180/pi;
	plot (...
	     t, z, GUP.ltype{rmb} ...
	     )
        hold on
    end    
    if ~oldhold, hold off, end
    annot('time [ \mus ]','phase []','',rtgconst('chmean'))  
    
elseif ID == 9.5 %}{-------------------[ phase tx ]------------------------
		 % only used by tau0 experiment, 14/10/99, Am

    oldhold = ishold;
    linetype = {'y-o','c-o'};
    N = GUP.Nfract*GUP.ACnbits;
    a = 1:N;
    for rmb = RMB;
        [z,t] = tx(rmb);
        z = angle(z) * 180/pi;
        [vel, freq, flip] = get_doppler(t(a),z(a));
        for k = 1:2
	     plot (...
	         a-1, z(a), linetype{k} ...
	         )
            a = a + N;
            hold on
        end
    end    
    if ~oldhold, hold off, end
    grid
    annot('Addr', 'Phase []', ...
            sprintf(' D=%1.1f m/s  \\Delta\\Phi=%d', vel, round(flip)), ...
            rtgconst('chmean'));



elseif ID >= 10 & ID <= 13 %}{--------------[ acf | spec ]---------------
test=rtg_CHANNELS(RMB(1));   % 12-Oct-1999 Am: added to suppress function isLP
			     % in order to provide a pp_generic

    if ismulti(RMB) 
        pperr('multiple channels not allowed (now)'), return
    end

    %----------------------------------------------------------------
    % Compute acf range gates to a matrix, with columns as the gates
    %----------------------------------------------------------------

    if test.isLP
        ACF = lp_acfmatrix( RMB, GATE.value, ismember(ID,GUP.bsubmodes) );
    else %altcode
        ACF = ac_acfmatrix(GATE.value, GUP.ssa(RMB), GUP.ACnbits,...
           GUP.AClags(end), GUP.Nfract, GUP.Nsamples, GUP.Lagweight);
        
    end       

    GATE.value = GATE.value(1:size(ACF,2));   % original GATEs may go to far
    range = gaterange(GATE.value,RMB(1)); 

    %-------------------------------------
    % Perform range scaling if applicable
    %-------------------------------------

    % Note the strategy: We could apply range scaling to altcodes always,
    % but do it here only if we are in "SPEC (-b)" mode. 

    if strcmp(ppopt('range_scaling'),'yes') & ismember(ID,GUP.bsubmodes);
        ACF = ACF .* kron(ones(size(ACF,1),1), (range .^ 2) / 100^2);
        wtype = 'r^2';
    else
        wtype = '';    
    end        

    ngates= size(ACF,2);

    %----------------------------------
    % Unbiase the acf(s) if requested :
    %----------------------------------

    if test.isLP
        maxlag = GUP.LPlags(end);
    else
        maxlag = GUP.AClags(end);  
    end           

    if strcmp( ppopt('biased_ACF'), 'yes')
        ACF = ACF / (maxlag+1);
        wtype= [wtype 'b'];
    else
        weight = 1 ./ ( (maxlag+1) - (0:maxlag)' );
        ACF = ACF .* kron( ones(1,ngates), weight);
        wtype= [wtype 'u'];
    end        

    %--------------
    % Plot acf(s) :
    %--------------


	%%%%%%%%%%%%%%%%%%
	nlags=size(ACF,1);
   	lag = (0:nlags-1)*GUP.Tau*1e6;
	%%%%%%%%%%%%%%%%%%


    if ismember(ID, GUP.acfmodes) %{

        if ngates <= 5
            rtg_userfig2('delete');
			   acf_lineplot(ACF,lag);
	         zoomcheck('auto');

        else %ngates > 5
            type = ppopt('acfplottype');
            
            if strcmp(type,'pcolor')		
		   	   acf_pcolor(real(ACF),range,lag);

            elseif strcmp(type,'mesh')
	                   acf_meshplot(real(ACF),range,lag)
	
            elseif strcmp(type,'image')
			   pp_imageplot(real(ACF'),range,lag,'Lag [\mus]')     

            elseif strcmp(type,'contour')
                      pp_contourplot(real(ACF'),range,lag);
            end

            rtg_userfig2('init');
            t = 0:(size(ACF,1)-1);            
            rtg_waterf(t,real(ACF));
            
        end           
    end %}

    %---------------------------
    % Compute and plot spectra :
    %---------------------------

    if ismember(ID, GUP.spemodes)
        flim = ppopt('f');
	if flim
	    fmin = flim(1);
            fmax = flim(2);
        else
            fmin = -Inf;
            fmax = +Inf;
        end

        usewin = strcmp( ppopt('windowed'), 'yes');
        [spe,freq] = acfspec( ACF, 0, GUP.Fs, fmin, fmax, usewin, 0,0);

        freq= freq/1000; %freq in kHz
        if length(GATE.value) <= 5
            rtg_userfig2('delete');
            spe_lineplot(freq,spe);
            zoomcheck('auto');
        else
            pp_imageplot( real(spe'),range, freq,'Frequency [kHz]');
            rtg_userfig2('init');
            rtg_waterf(freq,spe);
        end
        if usewin
            wtype= [wtype,'H'];
        else    
            wtype= [wtype,'n'];
        end
    end

    % Draw header

    acfsp_header(RMB,GATE.value,range, wtype);

else %}{--------------------- [ else ]------------------------------------

    pperr('Illegal MODE');
    
end %}MODES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    return
    rtg_debug(2,sprintf('  %s done',mfilename));

%}pp_generic


%#############################################################################
%                       SUBFUNCTIONS                                        
%#############################################################################
%                       annot           (xlbl,ylbl,extra,ch_ave)
%     A =               
%                       acfsp_header    (RMB, gate, range, wtype)
%                       acf_lineplot    (ACF,RMB)
%                       acf_meshplot    (A,range,RMB)
%                       acf_pcolor      (ACF,time,range,lagindex)
%     [psa,plen] =      ac_profmap	now defined in xxxx_gdef.m
%					Am, 12-Oct-1999
%     bgr =             bpp             (p,RMB)
%     cal=              calpp           (p,RMB)
%     [isnew,version] = check_version
%     r=                gaterange       (gates,RMB)
%     [gupid,version] = getversion      (s)
%                       imageplot       (C, ncolors, range, x, xlbl)
%     s =               ismulti         (RMB)
%     A= 		lp_acfmatrix    (RMB, gates, subtract)
%     [psa,plen] =      lp_profmap      now defined in gup3_gdef.m
%					Am, 12-Oct-1999
%     r =               ppr             (RMB)
%     scaler =          ppscaler        (RMB,r)
%                       pp_init	        now defined in gup3_gdef.m
%					Am, 12-Oct-1999			
%     ok =              pp_reinit       (version), now defined in gup3_gdef.m
%					Am, 12-Oct-1999
%     [rmb,id]=         pp_xxx_checks
%                       pperr          (msg)
%                       spe_lineplot    (freq,spectra)
%     sig =             spp             (RMB)
%     [z,t] =           tx              (rmb)
%			zoomcheck	(param)
%=============================================================================


function annot(xlbl,ylbl,extra,ch_ave) %{
%-------------------------------------
    global GUP rtg_USERAX MODE DMPT rtg_CHANNELS FREQCH
    if nargin ~= 4
        error('Wrong number of arguments')
    end        
    if ~isempty(rtg_USERAX)
        axis(rtg_USERAX); 
    else  
        axis('auto'); 
    end     
    xax; 
    xlabel(xlbl);
    ylabel(ylbl);
    RMB = rtg_CHANNELS(FREQCH).RMB;
    ave_mode = (ch_ave & length(RMB) == 2);
    rtguh(RMB, MODE, GUP.f(RMB), fix(DMPT), extra, ave_mode);       
%}annot


function s = ismulti(RMB) %{
%------------------------
    s = length(RMB) > 2;
%}ismulti    

function sig = spp(RMB) %{
%----------------------
global GUP d_data rtg_CHANNELS

test=rtg_CHANNELS(RMB(1));   % 12-Oct-1999 Am: added to suppress function isLP
			     % in order to provide a pp_generic

    if test.isLP
        s_adr = GUP.ssa(RMB(1)) + GUP.pp;
    else
        s_adr = GUP.ssa(RMB(1)) + GUP.pp(1:end-1);
    end        
    sig = real(d_data(s_adr))/GUP.sbweight;
					%  2-Jul-1999 Jm: dvision by 4 removed
    				        % 12-Oct-1999 Am: GUP.sbweight created
    					% equals 4 for GUP0, 1 otherwise
	
    if length(RMB) > 1 						
       sig= 0.5*( sig + spp(RMB(2)) );
    end        
%}spp


function bgr = bpp(p,RMB) %{
%-----------------------
global GUP d_data rtg_CHANNELS

		 test=rtg_CHANNELS(RMB(1));   % cf. spp

  if p==1
    if test.isLP
        adr= GUP.b1sa(RMB(1)) + GUP.pp;    
    else
        adr= GUP.b1sa(RMB(1)) + GUP.pp(1:end-1);                        
     end
  else
     adr= GUP.b2sa(RMB(1)) + GUP.pp;
  end  
     
    bgr= real(d_data(adr));
    if length(RMB) > 1, bgr= 0.5*( bgr + bpp(p,RMB(2)) ); end            
%}bpp    


function cal= calpp(p,RMB) %{
%-------------------------
global GUP d_data
if p==1
   adr= GUP.c1sa(RMB(1)) + GUP.cpp;
else
   adr= GUP.c2sa(RMB(1)) + GUP.cpp;
end
    cal= real(d_data(adr));
if length(RMB) > 1, cal= 0.5*( cal + calpp(2,RMB(2)) ); end        
%}calpp     


function r = ppr(RMB) %{
%---------------------
% Comments about GUP3
%	Ranges for minimum lag profiles.
% * For LP, we start ranges from the nomical middle point of illumination of
%   first sample, with no filter delays etc.
% * For AC, we use the nominal top edge of the lowest lag-1/2 gate, which is
%   3*Tau above the bottom of the illumination of the first sample.
% * Also note that the AC lag-1/2 profile contains only nsamples-1 points.
%
% 2-July-1999 Jm

global GUP rtg_CHANNELS
test=rtg_CHANNELS(RMB(1));   % cf. spp
dr = GUP.dr; 
                             
    if test.isLP	
        pulselen = GUP.LPlen(RMB(1));  		
        r = (GUP.pp-1)*dr + GUP.r0(RMB(1)) - (pulselen/2)*0.15; 
    else
        nbits = GUP.ACnbits;
        % Range of the the top of the lowest lag-1/Nfract gate: 
        r0 = GUP.r0(RMB(1)) +dr - (nbits-1)*GUP.ACbaud*0.15; 
        r = r0 + (GUP.pp(1:end-1)-1)*dr;
    end					
%}ppr


function r = gaterange(gates,RMB)
%--------------------------------{
% See comments in the function ppr

global GUP rtg_CHANNELS
test=rtg_CHANNELS(RMB(1));   % cf. spp

    if test.isLP
        dr = GUP.dr;	     % cf. ppr	
        r0 = GUP.r0(RMB) - (GUP.LPlen(RMB)/2)*0.15;
    else		    		      
	dr = GUP.dr;         % cf. ppr 			
      	r0 = GUP.r0(RMB(1)); % - (2*(GUP.ACnbits-2)+1)*GUP.drLP; Jm remark
    end					
        r = round( r0 + (gates-1)*dr );
%}gaterange    



function [z,t] = tx(rmb) %{
%-----------------------
    global GUP d_data
    if GUP.txsamples
        atx = 1:GUP.tlen(rmb);
        a = GUP.tsa(rmb) + atx;
        z = d_data(a);
        t = (atx-1)*GUP.Tau*1e6; 
    else
        z = [];
        t = [];
        pperr('TX samples not available')
        return
    end        
%}tx    



function A = lp_acfmatrix(RMB, gates, subtract) %{
%----------------------------------------------
    global GUP rtg_CHANNELS
    
    lags  = GUP.LPlags; 			
    profsa = GUP.LPprofsa;
    proflen = GUP.LPproflen;
    
    for k= 1:length(RMB)
       sgprofsa(k,:)  = GUP.ssa(RMB(k)) + profsa;
       test=rtg_CHANNELS(RMB(k));   % cf. spp
       if test.isLP
          b1profsa(k,:) = GUP.b1sa(RMB(k)) + profsa;
          if exist('GUP.b2sa'), b2profsa(k,:) = GUP.b2sa(RMB(k)) + profsa; end
       end
    end
     
    gates = round(gates);

    if gates(end) > proflen + profsa - length(lags)
        gt_ok = find(gates <= proflen + profsa - length(lags));
        gates = gates(gt_ok);
    end 
      
    A = zeros(length(lags),length(gates));

    for k = 1:length(gates)
        gt = gates(k);
        sacf = acfgate(sgprofsa, gt, length(lags));
        if subtract
           if exist('GUP.b2sa')
           bacf= 0.5 * ( acfgate(b1profsa,gt,length(lags),1) ...
                        + acfgate(b2profsa,gt,length(lags),1) );   
           else
              bacf = acfgate(b1profsa,gt,length(lags),1);
           end   
            acf = sacf/GUP.sbweight - bacf;
        else
            acf = sacf/GUP.sbweight;
        end
       	A(:,k)= acf;
    end
%}lp_acfmatrix



function acf_lineplot(ACF,lag) %{
%-----------------------------
    global GUP pp_OPT
    oldhold= ishold;
    ngates= size(ACF,2);    
    t =lag;

    if ngates == 1
        plot( t, real(ACF), 'yo',  ...
              t, real(ACF), 'm-',  ...
              t, imag(ACF), 'c-' )
    else
        for k= 1:ngates
            c= pp_OPT.gatecolor{k};
            plot( t,real(ACF(:,k)), [c '-'],  ...
                  t,imag(ACF(:,k)), 'w:' )
            hold on
        end
    end
    if ~oldhold, hold off, end    
    xax;    
    xlabel('Lag [\mus]');     
%}acf_lineplot




function spe_lineplot(freq,spectra) %{
%----------------------------------
    global pp_OPT
    oldhold = ishold;
    if size(spectra,2) == 1
        plot(freq, spectra, 'm-');
    else
        for gate = 1:size(spectra,2)
            plot(freq, spectra(:,gate), pp_OPT.gatecolor{gate});
            hold on
        end
    end
    if ~oldhold, hold off, end    
    grid
    xlabel('kHz')
%}spe_lineplot


function acfsp_header(RMB, gate, range, wtype) %{
%---------------------------------------------
    global MODE GUP rtg_FIGhandle DMPT pp_OPT
    if nargin ~= 4, error('wrong number of arguments'), end
    gate= round(gate);
    set(0,'CurrentFigure',rtg_FIGhandle(2));
    if length(gate) == 1
        gate_str= sprintf('G %g R %g W %s', gate, range, wtype); 
    elseif length(gate) <= 5
        gate_list=  ['[' deblank(sprintf('%g ',gate)) ']' ];
        range_list= ['[' deblank(sprintf('%g ',range)) ']' ];
        gate_str= sprintf('G%s R%s W %s', gate_list, range_list, wtype);
    else
        gate_list=   sprintf('[%g-%g]',min(gate),max(gate));
        range_list=  sprintf('[%g-%g]',min(range),max(range));
        gate_str= sprintf('G%s R%s W %s', gate_list, range_list, wtype);
    end
    ch_ave= length(RMB) > 0;
    rtguh( RMB, MODE, [], fix(DMPT), gate_str, ch_ave);
%}acfsp_header




function scaler = ppscaler(RMB,r) %{
%--------------------------------
    global GUP pp_OPT GAIN
    
    if strcmp(pp_OPT.range_scaling,'yes')
        r0= 100;
        scaler = (GUP.weight(RMB)/r0^2) .* (r.^2)';
    else    
        scaler = GUP.weight(RMB);
    end
    if strcmp(pp_OPT.gain_scaling, 'yes') 
        if ~isempty(GAIN) 
            scaler = scaler ./ GAIN';
        else   
            pperr('GAIN data file gainraw.mat not found');
            pp_OPT.gain_scaling = 'no';
        end    
    end    
%}ppscaler  


function [rmb,id] = pp_xxx_checks
%{-------------------------------
    global rtg_USERAX rtg_CHANNELS rtg_MODES d_data MODE FREQCH
    if isempty(MODE)
        disp('MODE = [] -- using 1')
        MODE = 1;
    end   
    if isempty(FREQCH)
        disp('FREQCH = [] -- using 1')
        FREQCH = 1;
    end   

    rmb = rtg_CHANNELS(FREQCH).RMB;
    id = rtg_MODES(MODE).id;

    if isempty(d_data)
        rmb = [];
        pperr('no data');
    end    
    return;
%}pp_xxx_checks


function zoomcheck(param)
%------------------------{
    global rtg_USERAX
    if ~isempty(rtg_USERAX)
        axis(rtg_USERAX); 
    else 
        eval( [ 'axis ' param ] );
    end     
%}zoomcheck
    
function x = xpp(rmb)
%--------------------
% Only usefull for TAU0, 14/10/99, Am
    global d_data
    [a,len] = levelmap(rmb,1);
    x = real(d_data(a+1:a+len));
%}xpp    


function [w,tail] = wpp(rmb)
%---------------------------
% Only usefull for TAU0, 14/10/99, Am
% Compute the weight of the lag 1/Nfract profile. Assume here that
% the nkill "deweighting" has NOT been done by decodump. 
%{
    global GUP
    nfract = GUP.Nfract;
    nbits = GUP.ACnbits;
    proflen = GUP.Nsamples-1;
    w = ones(proflen,1);
    if GUP.Nkill(rmb) > 0
      nkill = GUP.Nkill(rmb) - 1;% number of weight 2 points in lowest lag prof
        w(1:nkill) = 2;
    else
        nkill = 0;
    end        
    h = ones(nbits-1,1);
    h = insertz(h,nfract-1);
    w = filter(h,1,w);
    tailend = min(proflen,nkill+2*(nbits-2));
    tail = 1: tailend;   
    fig = gcf;
%    getfigure('A');
%    plot(w,'-o')    
%    set(0,'currentfigure',fig)
%}wpp


function [v,df,flipp] = get_doppler(t,pha)
%-----------------------------------------
% Only usefull for TAU0, 14/10/99, Am
%{
    D = abs(diff(pha));
    flipps = find(D>100);
    if isempty(flipps)
        flipp = 0;
        v = 0;
        df = 0;
        return;
    end        
    flipp = mean(D(flipps));
    pos = find(pha>0);
    neg = find(pha<0);
    P = pha(pos);
    N = pha(neg);
    if isempty(P)
        Y = N;
        X = t(neg);
    elseif isempty(N)
        Y = P;
        X = t(pos);
    else                
        if mean(P) > mean(abs(N))
            Y = P;
            X = t(pos);
        else
            Y = N;
            X = t(neg);
        end                    
    end
    poly = polyfit(X(:),Y(:),1);
    df = poly(1)*1e6/360;   % in Hz, when t is in usec
    % df/f = 2v/c ==>
    v = df *0.3; % m/s
%}        


function is = isLP(RMB)
%----------------------
%{
    global rtg_CHANNELS
    test = rtg_CHANNELS(RMB(1));
    is = test.isLP ~= 0;
%}    
