function pp_sethate(Rmb,Elev,varargin)

%PP_SETHATE Set up range gating by specifying height gates for a given elevation.
%   PP_SETHATE (RMB, ELEV, HIGHT_LIST, GATINGFACTOR)
%   PP_SETHATE (RMB, ELEV, HIGHT1, HIGHT2, GATINGFACTOR)
%
%   ELEV is in degrees, heights in kilometers. The GATINGFACTOR tells
%   how many adjacent raw ACF range gates are averaged together to form
%   an output gate.
%
%   Note that experiment specs must be available in the global
%   variable GUP (because the gating info is stored -- in the global GATE -- 
%   as gate numbers, and that requires knowledge about start ranges and
%   so on).
%
%   Note also that for alternating codes, only the 'fully weighted' gates
%   are accepted -- lower gates are silently ignored.
%   
%   See also pp_setgate.
%
%   4-Dec-1999 Jm    

    if nargin == 4

        if isempty(Elev)
            global ELEV
            Elev = ELEV;
        end            

        if Elev > 90
            error('illegal elevation');
        end            

        H = varargin{1};            % Vector of gate ranges
        R = alt2range(H,Elev);
        Gating = varargin{2};       % Gating factor
        pp_setgate(Rmb,R,Gating);
        
    elseif nargin == 5

        H1 = varargin{1};           % Range of first gate
        H2 = varargin{2};           % Range of last gate
        Gating = varargin{3};       % Gating factor
        R1 = alt2range(H1,Elev);
        R2 = alt2range(H2,Elev);
        pp_setgate(Rmb,R1,R2,Gating);

    else
        error('Wrong number of input arguments')
    end 
